/* written by John Doug Reynolds, June 2002 */
//
//   CHANGE HISTORY:
//       AI, 10-06-2004: #include<tabgtigen.h> must be included in order to solve
//                       an error in the mac exection.
//       AI, 06-07-2004: New rgsbkgmodel method introduced. Now rgsproc can produce
//                       rgs background model file.
//       AI, 16-05-2006: New parameters "pdistincl" added in the call to
//                       rgsbkgmodel
//
//

#include "rgsproc.h"
#include <metatask.h>
#include <OalServer.h>
#include <DalVec.h>
#include "OdfInfo.h"

#include <htrframes.h>
#include <atthkgen.h>
#include <attfilter.h>
#include <tabgtigen.h>
#include <hkgtigen.h>
#include <rgsoffsetcalc.h>
#include <rgssources.h>
#include <rgsframes.h>
#include <rgsenergy.h>
#include <rgsbadpix.h>
#include <rgsevents.h>
#include <evlistcomb.h>
#include <rgsangles.h>
#include <rgsfilter.h>
#include <rgsregions.h>
#include <rgsspectrum.h>
#include <rgsbkgmodel.h>
#include <rgsrmfgen.h>
#include <rgsfluxer.h>
#include <rgslccorr.h>
#include <heliocentriccorr.h>

#include <algorithm>
#include <unistd.h>
#include <cstdio>
#include <map>

//AI 10/06/04
// This definitions produces a exception in mac
//#define STRINGVECTOR(a) vector<string>(a,a+sizeof(a)/sizeof(*a))

namespace {
using std::string; using std::vector;

enum ProductType {
	ATTTSR, SRCLI_, EVENLI, EXPMAP, SRSPEC, BGSPEC,
	ATTGTI, HKGTI_, OFFSET, OFFHST, EVENTS, MERGED, RSPMAT, FLUXED, MBSPEC, SRTSR_, BGTSR_, SBTSR_,
	RMFMAT,ARFMAT
};

using namespace std;

class Machine {
public:
	// meta-task level parameters

	vector<int> orders;
	vector<string> stages, expunge;
	bool withprefix;
	string prefix;

	// Instrument data modes to check for SPECTROSCOPY_SMALL_WINDOW
	std::map<string , Xmm::Mode> _instrExpoModes ;

	// sub-task level parameters

	// atthkgen
	float timestep;
	// attfilter, tabgtigen
	bool onlylastatt;
	float driftlimit;
	// hkgtigen
	bool includeinputfifofull;
	// rgsoffsetcalc
	bool calcoffsets, withoffsethistogram;
	// rgssources
	string epicset, attstyle, srclabel, srcstyle;
	float attra, attdec, attapos, srcdisp, srcxdsp, srcra, srcdec, srcrate;
	bool withsrc, withepicset;
	// rgsenergy
	bool withcti, withgain, withdiagoffset;
	// rgsbadpix
	int pixnoiselimit, colnoiselimit;
	float pixsharpness, colsharpness;
        bool withadvisory, withfoundhot, keepcool, embadpixalgo;
        string badpixalgo;
	// rgsevents
	string detcoord;
	bool withdetcoord, reconstruct;
	// rgsangles
        bool withpointingcolumn, withmlambdacolumn;
        bool withheliocentriccorr,withsunanglecorr;
	float betabinref, betabinwidth, betamin, betamax;
	float xdispbinref, xdispbinwidth, xdispmin, xdispmax;
	string betabinning, xdispbinning;
	int nbetabins, nxdispbins;
	//AI: Wavelength scale. Start
	// New parameters to deal with the new lambda grid.
	double lambdabinref, lambdabinwidth;
	int nlambdabins;
	//AI: Wavelength scale. End
	// rgsfilter
	vector<string> rejflags, auxgtitables;
	float driftbinsize;
	bool withcombmap;
	// rgsregions
	int spatialres, orderres;
	float xpsfincl, xpsfexcl, pdistincl;
	string procsrcsexpr, exclsrcsexpr;
	bool withpicutoff;
	double picutoff;
	std::vector<double>  xdspregions;
	std::vector<double>  xdspbackground;
	// rgsspectrum
	bool bkgcorrect, withbkgset, withfracexp;
	int rebin, edgechannels, badquality;
	float exposed;
	//AI: Wavelength scale. Start
	string spectrumbinning;
	//AI: Wavelength scale. End
	// rgsbkgmodel
	bool withbackgroundmodel;
	float pdist;
	// rgsrmfgen
	int rmfbins, fftdim;
	bool withmirrorpsf, withangdist, witharffile;
	string angdistset;
	// rgsfluxer
	vector<int> flxquality;
	string flxmode, flxformat;
	bool mergeorders, withflxnan;
	//AI: New effective area correction
	bool withdyneffarea, withrectification,witheffectiveareacorrection;
	float flxmin, flxmax, flxnan;
	int flxbins;
	//rgslccorr
	double timeBinSize;
	double withbkgsubtraction;
	//heliocentric velocity correction
	bool withheliocorr;




	Machine();

	void pipeline( const string &instexpid ) const;
	void fluxing( const vector<string> &instexpids ) const;
	void lightcurve( const vector<string> &instexpids ) const;


private:
	std::map<enum ProductType,string> _glossary; // product type names

	void observationProc() const;
	void instrumentProc( enum Xmm::Instrument ) const;
	void exposureProc( const string &instexpid ) const;
	void ccdProc( const string &instexpid, int ccd ) const;

	void evlistcomb( const string &instexpid,
			const vector<string> &eventsets ) const;
	void rgsangles( const string &instexpid ) const;
	void rgsfilter( const string &instexpid ) const;
	void rgsregions( const string &instexpid ) const;
	void rgsspectrum( const string &instexpid, int source ) const;
	//heliocentric velocity correction
	void rgsheliocorr( const string fileName, const string srcList) const;
	void rgsbkgmodel( const string &instexpid) const;
	void rgsrmfgen( const string &instexpid, int order, int source ) const;
	void rgsfluxer( const vector<string> &pha,
			const vector<string> &rmf, const vector<string> &arf, const vector<string> &bkg, int order=0 ) const;

	void rgslccorr(const string &instexpid,const std::vector<std::string> &evt,
			const std::vector<std::string> &src,int order, int source, bool sw) const;

	string pname( enum ProductType, const string &instexpid,
			int degree=0, int source=0, const char *suffix=0 ) const;

	string pname( enum ProductType,
			enum Xmm::Instrument instrument=Xmm::INSTRUMENT_NONE,
			int degree=0, int source=0, const char *suffix=0 ) const;
};

bool find( const vector<string> &flags, const string &status )
{ return find(flags.begin(),flags.end(),status) != flags.end(); }
}

/* ------------------------------------------------------------------------- */

void Machine::pipeline( const string &instexpid ) const
{
	string merged = pname(MERGED,instexpid), evenli = pname(EVENLI,instexpid);

	if ( find(stages,"1:events") ) {
		observationProc();
		instrumentProc(InstrumentExposureIdentifier(instexpid).instrument());
		exposureProc(instexpid);


		vector<string> sets;
		for ( int ccd = 1; ccd <= 9; ++ccd ) {
			string ccdset = pname(EVENTS,instexpid,ccd);
			if ( dataSetServer->exists(ccdset) ) {
				ccdProc(instexpid,ccd);
				sets.push_back(ccdset);
			}
		}

		if ( sets.size() == 0 ) {
			errstr << instexpid << ": no SPE files in ODF"
			<< Sas::warning("SkippedExposure");
			return;
		}


		evlistcomb(instexpid,sets);

		if ( find(expunge,"events") ) {
			vector<string>::const_iterator itr;
			for ( itr = sets.begin(); itr != sets.end(); ++itr )
				unlink( itr->c_str() );
		}

	}

	if ( find(stages,"2:angles") || find(stages,"3:filter") ) {
		if ( !dataSetServer->exists(merged) )
		{
			if ( !dataSetServer->exists(evenli) ) {
				errstr << instexpid << ": no combined event list"
				<< Sas::warning("SkippedExposure");
				return;
			}
			else {
				errstr << instexpid << ": creating the missing combined event "
				<< "list as an identical copy of the existing filtered "
				<< "event list, thus possibly incurring more filtering "
				<< "than requested"
				<< Sas::warning("ExtraFiltering");
				struct DataSet* ds = dataSetServer->clone(evenli,merged,Dal::Read);
				dataSetServer->close(ds);
			}
		}

		if ( find(stages,"2:angles") ) rgsangles(instexpid);

		if ( find(stages,"3:filter") ) {
			rgsfilter(instexpid);
			if ( find(expunge,"merged") ) unlink( merged.c_str() );
		}
	}

	if ( find(stages,"4:spectra") ) {
		if ( !dataSetServer->exists(evenli) ) {
			errstr << instexpid << ": no filtered event list"
			<< Sas::warning("SkippedExposure");
			return;
		}
		rgsregions(instexpid);

		string dsname = pname(SRCLI_,instexpid);
		struct DataSet* srcset = dataSetServer->open(dsname,Dal::Read);
		const Table* stab = srcset->table("SRCLIST");
		{
			const DalVecI index( stab->column("INDEX") );
			const DalVecL process( stab->column("PROCESS") );
			for ( unsigned row = 0; row < stab->rows(); ++row )
				if ( process[row] ) rgsspectrum(instexpid,index[row]);
		}

		if ( withbackgroundmodel )  rgsbkgmodel(instexpid);

		dataSetServer->close(srcset);
	}
}

void Machine::fluxing( const vector<string> &instexpids ) const
{
	vector<string> phaList, rmfList,bkgList, arfList;
	vector<int>::const_iterator oi;

	bool smallwindow = false;
	for ( oi = orders.begin(); oi != orders.end(); ++oi ) {
		vector<string>::const_iterator ei;
		for ( ei = instexpids.begin(); ei != instexpids.end(); ++ei ) {
			string dsname = pname(SRCLI_,*ei);
			struct DataSet* srcset = dataSetServer->open(dsname,Dal::Read);
			const Table *stab = srcset->table("SRCLIST");
			int primesrc = stab->attribute("PRIMESRC")->asInt();
			dataSetServer->close(srcset);

			if ( dataSetServer->exists(pname(SRSPEC,*ei,*oi,primesrc)) ) {

				//AI: Disable background subtraction in case we have a SPECTROSCOPY SMALL WINDOW
				// exposure

				for(map<string,Xmm::Mode>::const_iterator k = _instrExpoModes.begin();
					k != _instrExpoModes.end(); ++k)
				{
					if ( (*k).first == *ei )
					{
						if ( (*k).second == Xmm::SPECTROSCOPY_SMALL_WINDOW)
						{
							errstr  << Xmm::toString((*k).second) << " mode detected for exposure "
							<< *ei << " and order " << *oi <<
							". We will automatically disable this exposure for rgsfluxer call"
							<< Sas::warning("Small_Window_Mode(rgsfluxer)");
							smallwindow = true;
						}
					}
				}
				phaList.push_back( pname(SRSPEC,*ei,*oi,primesrc) );
				if (witharffile)
				{
					rmfList.push_back( pname(RMFMAT,*ei,*oi,primesrc) );
					arfList.push_back( pname(ARFMAT,*ei,*oi,primesrc) );
				}
				else
					rmfList.push_back( pname(RSPMAT,*ei,*oi,primesrc) );
				bkgList.push_back( pname(BGSPEC,*ei,*oi,primesrc) );

				//AI: Disable background subtraction in case we have a SPECTROSCOPY SMALL WINDOW
				// exposure

				rgsrmfgen(*ei,*oi,primesrc);
			}
			else
				errstr << pname(SRSPEC,*ei,*oi,primesrc) << ": file not found"
				<< Sas::warning("SkippedSpectrum");
		}

		/*
		 * rgsfluxer bkg logic
		 */
		if ( !mergeorders && phaList.size() )
		{
			if (bkgcorrect || smallwindow )
				bkgList.clear();

			rgsfluxer(phaList,rmfList,arfList,bkgList,*oi);

			phaList.clear();
			rmfList.clear();
			arfList.clear();
			bkgList.clear();
		}

	}
	if ( mergeorders && phaList.size() ) rgsfluxer(phaList,rmfList,arfList,bkgList);
}



void Machine::lightcurve( const vector<string> &instexpids ) const
{

	vector<int>::const_iterator oi;
	vector<string> evtList, srcList;
	int primesrc = -1;
	for ( oi = orders.begin(); oi != orders.end(); ++oi ) {

		vector<string>::const_iterator ei;
		for ( ei = instexpids.begin(); ei != instexpids.end(); ++ei ) {
			bool smallwindow = false;
			if (primesrc == -1)
			{
				string dsname = pname(SRCLI_,*ei);
				struct DataSet* srcset = dataSetServer->open(dsname,Dal::Read);
				const Table *stab = srcset->table("SRCLIST");
				primesrc = stab->attribute("PRIMESRC")->asInt();
				dataSetServer->close(srcset);
			}
			if ( dataSetServer->exists(pname(SRSPEC,*ei,*oi,primesrc)) ) {
				evtList.push_back( pname(EVENLI,*ei) );
				srcList.push_back( pname(SRCLI_,*ei) );

				//AI: Disable background subtraction in case we have a SPECTROSCOPY SMALL WINDOW
				// exposure

				for(map<string,Xmm::Mode>::const_iterator k = _instrExpoModes.begin();
					k != _instrExpoModes.end(); ++k)
				{
					if ( (*k).first == *ei )
					{
						if ( (*k).second == Xmm::SPECTROSCOPY_SMALL_WINDOW)
						{
							errstr  << Xmm::toString((*k).second) << " mode detected for exposure "
							<< *ei << " and order " << *oi <<
							". We will automatically disable this exposure for rgsfluxer call"
							<< Sas::warning("Small_Window_Mode(rgsfluxer)");
							smallwindow = true;
						}
					}
				}

				rgslccorr(*ei,evtList,srcList,*oi,primesrc,smallwindow);
				evtList.clear();
				srcList.clear();
			}
			else
				errstr << pname(SRSPEC,*ei,*oi,primesrc) << ": file not found"
				<< Sas::warning("SkippedSpectrum");


		}
	}



}

void rgsproc()
{
	struct Machine m;

	if ( !m.withprefix ) {
		const Odf *odf = oalServer.odf();
		m.prefix = string("P") + odf->proposalId() + odf->observationId();
	}

	vector<string> instexpids;
	if ( !booleanParameter("withinstexpids") )
		for ( unsigned rgs = 1; rgs <= 2; ++rgs )
			for ( unsigned htr = 0; htr <= 1; ++htr ) {
				enum Xmm::DataMode datamode
				= htr ? Xmm::HIGH_TIME_RESOLUTION : Xmm::DATA_SPECTROSCOPY;

				enum Xmm::Instrument instrument = rgs==1 ? Xmm::RGS1 : Xmm::RGS2;
				unsigned count = oalServer.odf()->exposures(instrument,datamode);

				for ( unsigned exp = 1; exp <= count; ++exp ) {
					oalServer.instrument(instrument);
					oalServer.dataMode(datamode);
					oalServer.exposureIndex(exp);
					instexpids.push_back( oalServer.instrExposureId() );
					oalServer.instrument();
				}
			}
	else
		instexpids = stringListParameter("instexpids");


	vector<string>::iterator itr;
	for ( itr = instexpids.begin(); itr != instexpids.end(); ++itr ) {
		*itr = InstrumentExposureIdentifier(*itr).instrumentExposureId();
		m.pipeline(*itr);
	}



	if ( find(m.stages,"5:fluxing") ) m.fluxing(instexpids);

	//rgslccorr
	if (find(m.stages,"6:lightcurve") ) m.lightcurve(instexpids);

}

/* --- Machine methods ----------------------------------------------------- */

namespace {
void choiceError( const string &value,
		const string &param, const string &location )
{
	errstr << "surprised by --" << param << "=" << value
	<< " at '" << location << "'" << Sas::error("InternalError");
}
}

Machine::Machine()
{
	_glossary[ATTTSR] = "ATTTSR";	_glossary[SRCLI_] = "SRCLI_";
	_glossary[EVENLI] = "EVENLI";	_glossary[EXPMAP] = "EXPMAP";
	_glossary[SRSPEC] = "SRSPEC";	_glossary[BGSPEC] = "BGSPEC";
	_glossary[ATTGTI] = "attgti";	_glossary[HKGTI_] = "hkgti_";
	_glossary[OFFSET] = "offset";	_glossary[OFFHST] = "offhst";
	_glossary[EVENTS] = "events";	_glossary[MERGED] = "merged";
	_glossary[RSPMAT] = "RSPMAT";	_glossary[FLUXED] = "fluxed";
	_glossary[MBSPEC] = "MBSPEC";   _glossary[SRTSR_] = "SRTSR_";
	_glossary[BGTSR_] = "BGTSR_";   _glossary[SBTSR_] = "SBTSR_";
	_glossary[RMFMAT] = "RMFMAT";   _glossary[ARFMAT] = "ARFMAT";

	// load the task parameters

	string entrystage = stringParameter("entrystage");
	string finalstage = stringParameter("finalstage");
	{
		//AI 10/06/04
		// Due to run time errors in mac OS. We have commented this lines
		// and now the parameters have been created using a vector<string>
		// also the STINGVECTOR define has been removed.
		/*
       char *v[] = { "events", "angles", "filter", "spectra", "fluxing" };
       vector<string> dict = STRINGVECTOR(v);
		 */
		std::vector<std::string> v;
		v.push_back("1:events");
		v.push_back("2:angles");
		v.push_back("3:filter");
		v.push_back("4:spectra");
		v.push_back("5:fluxing");
		v.push_back("6:lightcurve");

		std::vector<std::string> dict;

		dict = v;
		//AI 10/06/04

		unsigned entry = find(dict.begin(),dict.end(),entrystage) - dict.begin();
		unsigned final = find(dict.begin(),dict.end(),finalstage) - dict.begin();

		if ( entry == dict.size() )
			choiceError(entrystage,"entrystage","Machine constructor");

		if ( final == dict.size() )
			choiceError(finalstage,"finalstage","Machine constructor");

		if ( final < entry )
			errstr << "[" << entrystage << "," << finalstage
			<< "] excludes all stages" << Sas::error("InvalidStageRange");


		for ( unsigned s = 0; s < dict.size(); ++s )
			if ( entry <= s && final >= s ) stages.push_back(dict[s]);
	}
	orders = intListParameter("orders");
	withprefix = booleanParameter("withprefix");
	if ( withprefix )
		prefix = stringParameter("prefix");
	expunge = stringListParameter("expunge");
	// atthkgen
	timestep = realParameter("timestep");
	// attfilter, tabgtigen
	onlylastatt = booleanParameter("onlylastatt");
	driftlimit = realParameter("driftlimit");
	// hkgtigen
	includeinputfifofull = booleanParameter("includeinputfifofull");
	// rgsoffsetcalc
	calcoffsets = booleanParameter("calcoffsets");
	if ( calcoffsets )
		withoffsethistogram = booleanParameter("withoffsethistogram");
	// rgssources
	withepicset = booleanParameter("withepicset");
	if ( withepicset )
		epicset = stringParameter("epicset");
	attstyle = stringParameter("attstyle");
	if ( attstyle == "user" ) {
		attra = realParameter("attra");
		attdec = realParameter("attdec");
		attapos = realParameter("attapos");
	}
	withsrc = booleanParameter("withsrc");
	if ( withsrc ) {
		srcstyle = stringParameter("srcstyle");
		if ( srcstyle == "radec" ) {
			srcra = realParameter("srcra");
			srcdec = realParameter("srcdec");
		}
		else if ( srcstyle == "wrtatt" ) {
			srcdisp = realParameter("srcdisp");
			srcxdsp = realParameter("srcxdsp");
		}
		else choiceError(srcstyle,"srcstyle","Machine constructor");
		srclabel = stringParameter("srclabel");
		srcrate = realParameter("srcrate");
	}
	// rgsenergy
	withcti = booleanParameter("withcti");
	withgain = booleanParameter("withgain");
	withdiagoffset = booleanParameter("withdiagoffset");
	// rgsbadpix
	withadvisory = booleanParameter("withadvisory");
	withfoundhot = booleanParameter("withfoundhot");
	keepcool = booleanParameter("keepcool");
	embadpixalgo = booleanParameter("embadpixalgo");
	if ( withfoundhot ) {
		pixnoiselimit = intParameter("pixnoiselimit");
		colnoiselimit = intParameter("colnoiselimit");
		pixsharpness = realParameter("pixsharpness");
		colsharpness = realParameter("colsharpness");
	}
	// rgsevents
	withdetcoord = booleanParameter("withdetcoord");
	if ( withdetcoord )
		detcoord = stringParameter("detcoord");
	reconstruct = booleanParameter("reconstruct");
	// rgsangles
	betabinning = stringParameter("betabinning");
	//AI: Wavelength scale. Start
	//New lamdda parameters
	lambdabinref = realParameter("lambdabinref");
	lambdabinwidth = realParameter("lambdabinwidth");
	nlambdabins = intParameter("nlambdabins");
	//AI: Wavelength scale. Stop
	if ( betabinning == "binSize" ) {
		betabinref = realParameter("betabinref");
		betabinwidth = realParameter("betabinwidth");
	}
	else if ( betabinning == "Range" ) {
		betamin = realParameter("betamin");
		betamax = realParameter("betamax");
	}
	else choiceError(betabinning,"betabinning","Machine constructor");
	nbetabins = intParameter("nbetabins");
	xdispbinning = stringParameter("xdispbinning");
	if ( xdispbinning == "binSize" ) {
		xdispbinref = realParameter("xdispbinref");
		xdispbinwidth = realParameter("xdispbinwidth");
	}
	else if ( xdispbinning == "Range" ) {
		xdispmin = realParameter("xdispmin");
		xdispmax = realParameter("xdispmax");
	}
	else choiceError(xdispbinning,"xdispbinning","Machine constructor");
	nxdispbins = intParameter("nxdispbins");
	withpointingcolumn = booleanParameter("withpointingcolumn");
	withmlambdacolumn = booleanParameter("withmlambdacolumn");
	withheliocentriccorr = booleanParameter("withheliocentriccorr");
	withsunanglecorr = booleanParameter("withsunanglecorr");
	// rgsfilter
	rejflags = stringListParameter("rejflags");
	driftbinsize = realParameter("driftbinsize");
	auxgtitables = stringListParameter("auxgtitables");
	withcombmap = booleanParameter("withcombmap");
	// rgsregions
	spatialres = intParameter("spatialres");
	orderres = intParameter("orderres");
	xpsfincl = realParameter("xpsfincl");
	xpsfexcl = realParameter("xpsfexcl");
	pdistincl = realParameter("pdistincl");
	procsrcsexpr = stringParameter("procsrcsexpr");
	exclsrcsexpr = stringParameter("exclsrcsexpr");
	withpicutoff = booleanParameter("withpicutoff");
	picutoff = realParameter("picutoff");
	xdspregions = realListParameter("xdspregions");
	xdspbackground  = realListParameter("xdspbackground");
	// rgsspectrum
	bkgcorrect = booleanParameter("bkgcorrect");
	withbkgset = booleanParameter("withbkgset");
	withfracexp = booleanParameter("withfracexp");
	edgechannels = intParameter("edgechannels");
	rebin = intParameter("rebin");
	badquality = intParameter("badquality");
	exposed = realParameter("exposed");
	//heliocentric velocity correction
	withheliocorr = booleanParameter("withheliocorr");
	//AI: Wavelength scale. Start
	spectrumbinning =  stringParameter("spectrumbinning"); //Also valid for rgsrmfgen!!!
	//AI: Wavelength scale. End
	//rgsbkgmodel
	withbackgroundmodel = booleanParameter("withbackgroundmodel");
	pdist = realParameter("pdistincl");
	// rgsrmfgen
	witharffile = booleanParameter("witharffile");
	rmfbins = intParameter("rmfbins");
	fftdim = intParameter("fftdim");
	withmirrorpsf = booleanParameter("withmirrorpsf");
	withangdist = booleanParameter("withangdist");
	if ( withangdist )
		angdistset = stringParameter("angdistset");
	// rgsfluxer
	mergeorders = booleanParameter("mergeorders");
	flxformat = stringParameter("flxformat");
	flxmin = realParameter("flxmin");
	flxmax = realParameter("flxmax");
	flxbins = intParameter("flxbins");
	flxmode = stringParameter("flxmode");
	flxquality = intListParameter("flxquality");
	withflxnan = booleanParameter("withflxnan");
	withdyneffarea = booleanParameter("withdyneffarea");

	//rgslccorr
	timeBinSize = realParameter("timebinsize");
	withbkgsubtraction = booleanParameter("withbkgsubtraction");


	/*
	 * Empirical correction based on new EFFAREA extension
	 */
	withrectification = booleanParameter("withrectification");
	witheffectiveareacorrection = booleanParameter("witheffectiveareacorrection");
	if ( withflxnan )
		flxnan = realParameter("flxnan");
	/*
	 * Small Window mode identification
	 */
	for ( unsigned rgs = 1; rgs <= 2; ++rgs )
	{
		enum Xmm::Instrument instrument = rgs==1 ? Xmm::RGS1 : Xmm::RGS2;

		const OdfInfo::Odf *odf = OdfInfo::server->odf();
		const OdfInfo::ObservationExposureList& l = odf->observationExposures(instrument);
		for (OdfInfo::ObservationExposureList::const_iterator it=l.begin();
		       it != l.end();it++) {
			OdfInfo::ObservationExposure* e=(*it);
			_instrExpoModes[e->id()] = e->mode();
		}
	}

}

string Machine::pname( enum ProductType pt, const string &instexpid,
		int degree, int source, const char *suffix ) const
		{
	char degree_source[5];
	std::sprintf( degree_source, "%d%03X", degree, source );

	std::map<enum ProductType,string>::const_iterator itr = _glossary.find(pt);
	if ( itr == _glossary.end() )
		errstr << "unexpected ProductType code " << pt
		<< " for Machine::pname()" << Sas::error("InternalError");

	return prefix + instexpid +
	itr->second + degree_source + "." + (suffix ? suffix : "FIT");
		}

string Machine::pname( enum ProductType pt, enum Xmm::Instrument instrument,
		int degree, int source, const char *suffix ) const
{
	if ( instrument != Xmm::INSTRUMENT_NONE ) {
		string instexpid = Xmm::toString(instrument,true) + "X000";
		return pname( pt, instexpid, degree, source, suffix );
	}
	else
		return pname( pt, "OBX000", degree, source, suffix );
}

void Machine::observationProc() const
{
	string atthkset = pname(ATTTSR);
	if ( !dataSetServer->exists(atthkset) ) {
		openParameters("atthkgen");
		setParameter("timestep",timestep);
		setParameter("atthkset",atthkset);
		task(::atthkgen);
		closeParameters();
	}
}

void Machine::instrumentProc( enum Xmm::Instrument instrument ) const
{
	if ( calcoffsets ) {
		string outset = pname(OFFSET,instrument);
		if ( !dataSetServer->exists(outset) ) {
			vector<string> sets;
			const Odf *odf = oalServer.odf();
			oalServer.dataMode(Xmm::DATA_DIAGNOSTIC);
			for ( unsigned exp = 1; exp <= odf->exposures(); ++exp ) {
				oalServer.exposureIndex(exp);
				sets.push_back(odf->selectSet(Xmm::DATA_DIAGNOSTIC)->name());
			}

			openParameters("rgsoffsetcalc");
			setParameter("sets",sets);
			setParameter("outset",outset);
			setParameter("withhistogramset",withoffsethistogram);
			if ( withoffsethistogram )
				setParameter("histogramset",pname(OFFHST,instrument));
			task(::rgsoffsetcalc);
			closeParameters();
		}
	}
}

void Machine::exposureProc( const string &instexpid ) const
{
	enum Xmm::Instrument instrument =
		InstrumentExposureIdentifier(instexpid).instrument();

	char scratch[32];
	vector<string> tolangles;
	std::sprintf( scratch, "%f*#ARCMIN", driftlimit );
	tolangles.push_back(scratch);

	openParameters("attfilter");
	setParameter("filtertype","cone");
	setParameter("tolangles",tolangles);
	setParameter("instexpid",instexpid);
	setParameter("onlylastatt",onlylastatt);
	setParameter("attset",pname(ATTTSR));
	setParameter("gtitable",pname(ATTGTI,instexpid)+":STDGTI");
	task(::attfilter);
	closeParameters();


	openParameters("hkgtigen");
	// Switch to disable the INPUT FIFO FULL flags for bright sources.
	// As far as we have tested the events contained within INPUT FIFO FULL
	// time intervals are correct.
	// Then, we leave the user the possibility to disable this checking.

	if (includeinputfifofull)
	{

		setParameter("withparameters",true);
		setParameter("except",true);
		vector<string> r1param, r2param;
		r1param.push_back("G5063");
		r2param.push_back("L5063");
		switch ( instrument ) {
		case Xmm::RGS1: setParameter("parameters",r1param); break;
		case Xmm::RGS2: setParameter("parameters",r2param); break;
		default:
			errstr << "unexpected instrument " << Xmm::toString(instrument)
			<< " for Machine::exposureProc"
			<< Sas::error("InternalError");
		}
	}

	switch ( instrument ) {
	case Xmm::RGS1: setParameter("instrument","rgs1"); break;
	case Xmm::RGS2: setParameter("instrument","rgs2"); break;
	default:
		errstr << "unexpected instrument " << Xmm::toString(instrument)
		<< " for Machine::exposureProc"
		<< Sas::error("InternalError");
	}
	setParameter("gtiset",pname(HKGTI_,instexpid));
	task(::hkgtigen);
	closeParameters();

	openParameters("rgssources");
	setParameter("filemode","create");
	setParameter("instexpid",instexpid);
	setParameter("srclist",pname(SRCLI_,instexpid));
	setParameter("withepicset",withepicset);
	if ( withepicset ) {
		setParameter("epicset",epicset);
		setParameter("enablefilter",true);
	}
	setParameter("attstyle",attstyle);
	if ( attstyle == "user" ) {
		setParameter("attra",attra);
		setParameter("attdec",attdec);
		setParameter("attapos",attapos);
	}
	else if ( attstyle == "expmedian" )
		setParameter("expmediantable",pname(ATTGTI,instexpid)+":STDGTI");
	else if ( attstyle != "start" )
		choiceError(attstyle,"attstyle","Machine::exposureProc()");
	setParameter("addusersource",withsrc);
	if ( withsrc ) {
		setParameter("positionstyle",srcstyle);
		if ( srcstyle == "radec" ) {
			setParameter("ra",srcra);
			setParameter("dec",srcdec);
		}
		else if ( srcstyle == "wrtatt" ) {
			setParameter("deltadisp",srcdisp);
			setParameter("deltaxdsp",srcxdsp);
		}
		else choiceError(srcstyle,"srcstyle","Machine::exposureProc()");
		setParameter("userasprime",true);
		setParameter("label",srclabel);
		setParameter("rate",srcrate);
	}
	else
		setParameter("primestyle","auto");
	task(::rgssources);
	closeParameters();

	// meta-rgsframes, with htrframes-style interface and HTR mode support

	oalServer.instrExposureId( instexpid );
	if ( oalServer.dataMode() == Xmm::DATA_SPECTROSCOPY ) {
		const Exposure *exposure = oalServer.odf()-> currentExposure();

		const std::list<const OdfSet*> &odfs = exposure->sets();
		std::list<const OdfSet*>::const_iterator itr;
		for ( itr = odfs.begin(); itr != odfs.end(); ++itr )
			if ( (*itr)->type() == OdfSet::EVENT_LIST ) {
				openParameters("rgsframes");
				setParameter("ccdset",pname(EVENTS,instexpid,(*itr)->ccdNr()));
				setParameter("set",(*itr)->name());
				switch ( instrument ) {
					case Xmm::RGS1: setParameter("instrument","rgs1"); break;
					case Xmm::RGS2: setParameter("instrument","rgs2"); break;
					default:
						errstr << "unexpected instrument " << Xmm::toString(instrument)
						<< " for Machine::exposureProc"
						<< Sas::error("InternalError");
				}
				task(::rgsframes);
				closeParameters();
			}
	}
	else {
		openParameters("htrframes");
		setParameter("instexpid",instexpid);
		setParameter("prefix",prefix);
		task(::htrframes);
		closeParameters();
	}
}

void Machine::ccdProc( const string &instexpid, int ccd ) const
{
	string ccdset = pname(EVENTS,instexpid,ccd);

	openParameters("rgsenergy");
	setParameter("ccdset",ccdset);
	setParameter("withoffset",calcoffsets);
	if ( calcoffsets ) {
		enum Xmm::Instrument instrument
		= InstrumentExposureIdentifier(instexpid).instrument();
		setParameter("offset",pname(OFFSET,instrument));
	}
	setParameter("withcti",withcti);
	setParameter("withgain",withgain);
	setParameter("withdiagoffset",withdiagoffset);
	task(::rgsenergy);
	closeParameters();

	openParameters("rgsbadpix");
	setParameter("ccdset",ccdset);
	setParameter("withadvisory",withadvisory);
	setParameter("withfoundhot",withfoundhot);
	setParameter("keepcool",keepcool);
	setParameter("embadpixalgo",embadpixalgo);
	if ( withfoundhot ) {
		setParameter("pixnoiselimit",pixnoiselimit);
		setParameter("colnoiselimit",colnoiselimit);
		setParameter("pixsharpness",pixsharpness);
		setParameter("colsharpness",colsharpness);
	}
	task(::rgsbadpix);
	closeParameters();

	openParameters("rgsevents");
	setParameter("ccdset",ccdset);
	setParameter("reconstruct",reconstruct);
	setParameter("withdetcoord",withdetcoord);
	if ( withdetcoord )
		setParameter("detcoord",detcoord);
	task(::rgsevents);
	closeParameters();

}

void Machine::evlistcomb( const string &instexpid,
		const vector<string> &eventsets ) const
		{
	//AI 10/06/04
	// Due to run time errors in mac OS. We have commented this lines
	// and now the parameters have been created using a vector<string>
	// also the STINGVECTOR define has been removed.

	/*
    char *rgsdatamodes[] = {
    "SPECTROSCOPY", "HIGHTIMERESOLUTION"
    };
    char *mainattributes[] = {
    "CCDOCB", "DATE-OBS", "DATE-END", "DATATYPE",
    "TSTART", "TSTOP", "TELAPSE", "EXPIDSTR"
    };
    char *othertables[] = {
    "STDGTI", "BADPIX0", "BADPIX1", "REJPIX0", "REJPIX1", "EXPOSURE"
    };
    char *rgsimgcolnames[] = {
    "TIME", "FLAG", "BETA", "XDSP", "CHIPX", "CHIPY",
    "PHA", "SHAPE", "GRADE", "PI"
    };
    char *rgsimgcoltypes[] = {
    "double", "int32", "single", "single", "int16", "int16",
    "int16", "int8", "int8", "int16"
    };
    char *rgstimcolnames[] = {
    "TIME", "FLAG", "BETA", "XDSP", "CHIPX", "CHIPY", "PHA", "PI"
    };
    char *rgstimcoltypes[] = {
    "double", "int32", "single", "single", "int16", "int16", "int16", "int16"
    };

    oalServer.instrExposureId( instexpid );

   setParameter("instrument","rgs");
   setParameter("eventsets",eventsets);
   setParameter("rgsdatamodes",STRINGVECTOR(rgsdatamodes));
   setParameter("mainattributes",STRINGVECTOR(mainattributes));
   setParameter("othertables",STRINGVECTOR(othertables));
   if ( oalServer.dataMode() == Xmm::DATA_SPECTROSCOPY ) {
      setParameter("imagingset",pname(MERGED,instexpid));
      setParameter("rgsimgcolnames",STRINGVECTOR(rgsimgcolnames));
      setParameter("rgsimgcoltypes",STRINGVECTOR(rgsimgcoltypes));
   }
   else {
      setParameter("timingset",pname(MERGED,instexpid));
      setParameter("rgstimcolnames",STRINGVECTOR(rgstimcolnames));
      setParameter("rgstimcoltypes",STRINGVECTOR(rgstimcoltypes));
   }
	 */

	vector<string> rgsdatamodes;
	rgsdatamodes.push_back("SPECTROSCOPY");
	rgsdatamodes.push_back("HIGHTIMERESOLUTION");

	vector<string> mainattributes;
	mainattributes.push_back("CCDOCB");
	mainattributes.push_back("DATE-OBS");
	mainattributes.push_back("DATE-END");
	mainattributes.push_back("DATATYPE");
	mainattributes.push_back("TSTART");
	mainattributes.push_back("TSTOP");
	mainattributes.push_back("TELAPSE");
	mainattributes.push_back("EXPIDSTR");

	vector<string> othertables;
	othertables.push_back("STDGTI");
	othertables.push_back("BADPIX0");
	othertables.push_back("BADPIX1");
	othertables.push_back("REJPIX0");
	othertables.push_back("REJPIX1");
	othertables.push_back("EXPOSURE");

	vector<string> rgsimgcolnames;
	rgsimgcolnames.push_back("TIME");
	rgsimgcolnames.push_back("FLAG");
	rgsimgcolnames.push_back("BETA");
	rgsimgcolnames.push_back("XDSP");
	rgsimgcolnames.push_back("CHIPX");
	rgsimgcolnames.push_back("CHIPY");
	rgsimgcolnames.push_back("PHA");
	rgsimgcolnames.push_back("SHAPE");
	rgsimgcolnames.push_back("GRADE");
	rgsimgcolnames.push_back("PI");

	vector<string> rgsimgcoltypes;
	rgsimgcoltypes.push_back("double");
	rgsimgcoltypes.push_back("int32");
	rgsimgcoltypes.push_back("single");
	rgsimgcoltypes.push_back("single");
	rgsimgcoltypes.push_back("int16");
	rgsimgcoltypes.push_back("int16");
	rgsimgcoltypes.push_back("int16");
	rgsimgcoltypes.push_back("int8");
	rgsimgcoltypes.push_back("int8");
	rgsimgcoltypes.push_back("int16");


	vector<string> rgstimcolnames;
	rgstimcolnames.push_back("TIME");
	rgstimcolnames.push_back("FLAG");
	rgstimcolnames.push_back("BETA");
	rgstimcolnames.push_back("XDSP");
	rgstimcolnames.push_back("CHIPX");
	rgstimcolnames.push_back("CHIPY");
	rgstimcolnames.push_back("PHA");
	rgstimcolnames.push_back("PI");


	vector<string> rgstimcoltypes;
	rgstimcoltypes.push_back("double");
	rgstimcoltypes.push_back("int32");
	rgstimcoltypes.push_back("single");
	rgstimcoltypes.push_back("single");
	rgstimcoltypes.push_back("int16");
	rgstimcoltypes.push_back("int16");
	rgstimcoltypes.push_back("int16");
	rgstimcoltypes.push_back("int16");

	oalServer.instrExposureId( instexpid );

	openParameters("evlistcomb");
	setParameter("instrument","rgs");
	setParameter("eventsets",eventsets);
	setParameter("rgsdatamodes",rgsdatamodes);
	setParameter("mainattributes",mainattributes);
	setParameter("othertables",othertables);
	if ( oalServer.dataMode() == Xmm::DATA_SPECTROSCOPY ) {
		setParameter("imagingset",pname(MERGED,instexpid));
		setParameter("rgsimgcolnames",rgsimgcolnames);
		setParameter("rgsimgcoltypes",rgsimgcoltypes);
	}
	else {
		setParameter("timingset",pname(MERGED,instexpid));
		setParameter("rgstimcolnames",rgstimcolnames);
		setParameter("rgstimcoltypes",rgstimcoltypes);
	}
	//AI 10/06/04

	task(::evlistcomb);
	closeParameters();
		}

void Machine::rgsangles( const string &instexpid ) const
{
	openParameters("rgsangles");
	setParameter("srclist",pname(SRCLI_,instexpid));
	setParameter("mergedset",pname(MERGED,instexpid));
	setParameter("betabinning",betabinning);

	if ( betabinning == "binSize" ) {
		setParameter("betabinref",betabinref);
		setParameter("betabinwidth",betabinwidth);
	}
	else if ( betabinning == "Range" ) {
		setParameter("betamin",betamin);
		setParameter("betamax",betamax);
	}
	else choiceError(betabinning,"betabinning","Machine::rgsangles()");

	//AI: Wavelength scale. Start
	//New lamdda parameters
	setParameter("lambdabinref",lambdabinref);
	setParameter("lambdabinwidth",lambdabinwidth);
	setParameter("nlambdabins",nlambdabins);
	//AI: Wavelength scale. Stop


	setParameter("nbetabins",nbetabins);
	setParameter("xdispbinning",xdispbinning);
	if ( xdispbinning == "binSize" ) {
		setParameter("xdispbinref",xdispbinref);
		setParameter("xdispbinwidth",xdispbinwidth);
	}
	else if ( xdispbinning == "Range" ) {
		setParameter("xdispmin",xdispmin);
		setParameter("xdispmax",xdispmax);
	}
	else choiceError(xdispbinning,"xdispbinning","Machine::rgsangles()");
	setParameter("nxdispbins",nxdispbins);
	setParameter("withpointingcolumn",withpointingcolumn);
	setParameter("withmlambdacolumn",withmlambdacolumn);
	setParameter("withheliocentriccorr",withheliocentriccorr);
	setParameter("withsunanglecorr",withsunanglecorr);
	task(::rgsangles);
	closeParameters();
}

void Machine::rgsfilter( const string &instexpid ) const
{
	string attGTI = pname(ATTGTI,instexpid), hkGTI = pname(HKGTI_,instexpid);

	vector<string> gtitables = auxgtitables;
	if ( dataSetServer->exists(attGTI) )
		gtitables.push_back( attGTI + ":STDGTI" );
	if ( dataSetServer->exists(hkGTI) )
		gtitables.push_back( hkGTI + ":STDGTI" );

	openParameters("rgsfilter");
	setParameter("mergedset",pname(MERGED,instexpid));
	setParameter("evlist",pname(EVENLI,instexpid));
	setParameter("rejflags",rejflags);
	setParameter("driftbinsize",driftbinsize);
	setParameter("auxgtitables",gtitables);
	setParameter("withexpmaps",true);
	setParameter("withcombmap",withcombmap);
	setParameter("spectrumbinning",spectrumbinning);
	if ( withcombmap )
		setParameter("expimageset",pname(EXPMAP,instexpid));
	task(::rgsfilter);
	closeParameters();
}

void Machine::rgsregions( const string &instexpid ) const
{
	int maxOrder = 0;
	vector<int>::const_iterator oi;
	for ( oi = orders.begin(); oi != orders.end(); ++oi )
		if ( *oi > maxOrder ) maxOrder = *oi;

	openParameters("rgsregions");
	setParameter("spatialres",spatialres);
	setParameter("orderres",orderres);
	setParameter("xpsfbelow",xpsfincl);
	setParameter("xpsfabove",xpsfincl);
	setParameter("xpsfexcl",xpsfexcl);
	setParameter("pdistincl",pdistincl);
	setParameter("procsrcsexpr",procsrcsexpr);
	setParameter("exclsrcsexpr",exclsrcsexpr);
	setParameter("setflags",true);
	setParameter("maxorder",maxOrder);
	setParameter("evlist",pname(EVENLI,instexpid));
	setParameter("srclist",pname(SRCLI_,instexpid));
	setParameter("withpicutoff",withpicutoff);
	setParameter("picutoff",picutoff);
	setParameter("xdspregions",xdspregions);
	setParameter("xdspbackground",xdspbackground);
	setParameter("spectrumbinning",spectrumbinning);
	task(::rgsregions);
	closeParameters();

}

void Machine::rgsspectrum( const string &instexpid, int source ) const
{
	/*
	 * Disable the BKG creation for Small Window mode
	 */
	bool newwithbkgset = withbkgset;
	bool newbkgcorrect = bkgcorrect;


	vector<int>::const_iterator itr;
	for ( itr = orders.begin(); itr != orders.end(); ++itr ) {

		/*
		 * Search for small window exposure and skip the background creation
		 */
		for(map<string,Xmm::Mode>::const_iterator k = _instrExpoModes.begin();
			k != _instrExpoModes.end(); ++k)
		{
			if ( (*k).first == instexpid )
			{
				if ( (*k).second == Xmm::SPECTROSCOPY_SMALL_WINDOW)
				{
					errstr << Xmm::toString((*k).second) <<" mode detected for exposure " << instexpid
					<< " and order " << *itr <<". We will automatically disable the background generation "
							"and background subtraction options "
					<< Sas::warning("Small_Window_Mode(rgsspectrum)");
					newwithbkgset = false;
					newbkgcorrect = false;
				}
			}
		}
		/*
		 * Disable the BKG creation for Small Window mode
		 */


		openParameters("rgsspectrum");
		setParameter("withspectrum",true);
		setParameter("rebin",rebin);
		setParameter("edgechannels",edgechannels);
		setParameter("badquality",badquality);
		setParameter("exposed",exposed);
		setParameter("bkgcorrect",newbkgcorrect);
		setParameter("withbkgset",newwithbkgset);
		setParameter("withfracexp",withfracexp);
		setParameter("srclist",pname(SRCLI_,instexpid));
		setParameter("source",source);
		setParameter("order",*itr);
		setParameter("evlist",pname(EVENLI,instexpid));
		setParameter("spectrumset",pname(SRSPEC,instexpid,*itr,source));
		//AI: Wavelength scale. Start
		setParameter("spectrumbinning",spectrumbinning);
		//AI: Wavelength scale. End
		if ( newwithbkgset )
			setParameter("bkgset",pname(BGSPEC,instexpid,*itr,source));
		task(::rgsspectrum);

		closeParameters();

		if (withheliocorr)
		{
			rgsheliocorr(pname(SRSPEC,instexpid,*itr,source),pname(SRCLI_,instexpid));
			if ( newwithbkgset )
				rgsheliocorr(pname(BGSPEC,instexpid,*itr,source),pname(SRCLI_,instexpid));
		}
	}
}

void Machine::rgsheliocorr(std::string specName, std::string srcList ) const
{
	openParameters("heliocentriccorr");
	setParameter("spectrumset",specName);
	setParameter("srclist",srcList);
	task(::heliocentriccorr);
	closeParameters();
}

void Machine::rgsbkgmodel( const string &instexpid ) const
{
	// The following parameters should be set to the defaults values.
	errstr << "You have requested the background model option, "
	<< "This option needs to be run with the defaults values "
	<< "for the following parameter: betabinref, betabinwidth, "
	<< "nbetabins, bkgcorrect, rebin. Please, check your values "
	<< Sas::warning("Background model");

	vector<int>::const_iterator itr;
	//   std::string bckFile;
	//   std::string inst;

	for ( itr = orders.begin(); itr != orders.end(); ++itr ) {
		openParameters("rgsbkgmodel");
		setParameter("table",pname(EVENLI,instexpid));
		setParameter("order",*itr);
		setParameter("pdistincl",pdist);
		if (spectrumbinning == "lambda")
			setParameter("binningunits","LAMBDA");
		else
			setParameter("binningunits","BETA");
		setParameter("backgroundmodelset",pname(MBSPEC,instexpid,*itr));


		task(::rgsbkgmodel);
		closeParameters();
	}
}

void Machine::rgsrmfgen(const string &instexpid, int order, int source) const
{
	openParameters("rgsrmfgen");
	setParameter("newrmf",true);
	setParameter("withspectrum",true);
	setParameter("rows",rmfbins);
	setParameter("fftdim",fftdim);
	setParameter("withmirrorpsf",withmirrorpsf);
	setParameter("withangdist",withangdist);
	if ( withangdist )
		setParameter("angdistset",angdistset);
	setParameter("evlist",pname(EVENLI,instexpid));
	setParameter("srclist",pname(SRCLI_,instexpid));
	setParameter("spectrumset",pname(SRSPEC,instexpid,order,source));

	setParameter("witharffile",witharffile);
	if (witharffile)
	{
		setParameter("rmfset",pname(RMFMAT,instexpid,order,source));
		setParameter("arfset",pname(ARFMAT,instexpid,order,source));
	}
	else
		setParameter("rmfset",pname(RSPMAT,instexpid,order,source));
	//AI: Enable the new effective area correction
	setParameter("dyneffareacorr",withdyneffarea);
	setParameter("order",order);
	//AI: Enable or not the empirical rectification based on models...
	setParameter("withrectification",withrectification);
	setParameter("witheffectiveareacorrection",witheffectiveareacorrection);
	setParameter("spectrumbinning",spectrumbinning);
	task(::rgsrmfgen);
	closeParameters();
}

void Machine::rgsfluxer( const vector<string> &pha, const vector<string> &rmf,const vector<string> &arf, const vector<string> &bkg,
		int order ) const
{
	const char *suffix = 0;
	if      ( flxformat == "dal" )   suffix = 0;
	else if ( flxformat == "ascii" ) suffix = "txt";
	else if ( flxformat == "qdp" )   suffix = "qdp";
	else
		choiceError(flxformat,"flxformat","rgsfluxer()");

	openParameters("rgsfluxer");
	setParameter("min",flxmin);
	setParameter("max",flxmax);
	setParameter("bins",flxbins);
	setParameter("mode",flxmode);
	setParameter("quality",flxquality);
	setParameter("withaltnan",withflxnan);
	if ( withflxnan )
		setParameter("nan",flxnan);
	setParameter("format",flxformat);
	setParameter("pha",pha);
	setParameter("rmf",rmf);
	setParameter("bkg",bkg);

	if(arf.size() != 0)
	{
		setParameter("witharf",true);
		setParameter("arf",arf);
	}

	setParameter("file",pname(FLUXED,Xmm::INSTRUMENT_NONE,order,0,suffix));
	task(::rgsfluxer);
	closeParameters();
}

void Machine::rgslccorr(const string &instexpid,const std::vector<std::string> &evtlist,
		const std::vector<std::string> &srclist, int order, int source, bool smallwindow) const
{
	openParameters("rgslccorr");
	setParameter("evlist", evtlist);
	setParameter("srclist", srclist);
	setParameter("outputsrcfilename",pname(SRTSR_,instexpid,order,source));

	if ((smallwindow == true) || (withbkgsubtraction == false) )
	{
		setParameter("outputsrcfilename",pname(SBTSR_,instexpid,order,source));
		setParameter("withbkgsubtraction",false);
	}
	else
	{
		setParameter("withbkgsubtraction",true);
		setParameter("outputbkgfilename",pname(BGTSR_,instexpid,order,source));
	}
	setParameter("timebinsize",timeBinSize);
	vector<int> ord;
	ord.push_back(order);
	setParameter("orders",ord);
	std::stringstream s;
	s << source;

	setParameter("sourceid", s.str());
	task(::rgslccorr);
	closeParameters();
}
